#import "BufferListController.h"
#import "FBPrincipal.h"


@implementation BufferListController

- (void)awakeFromNib
{
    NSLog(@"%@ %s", [self className], _cmd);
    [[principal mediator] addPlugInView:bufferListScrollView withTitle:@"Buffer List"];
}

- (void)dealloc
{
    NSLog(@"%@ %s", [self className], _cmd);
    if (bufferList) {
        [bufferList release]; 
        bufferList = nil;
    }
    [bufferListScrollView release]; bufferListScrollView = nil;
    [super dealloc];
}

- (void)reloadBufferList
{
    if (bufferList) {
        [bufferList release];
        bufferList = nil;
    }

    NSString * expr = @"map(filter(range(1,bufnr('$')), 'buflisted(v:val) != 0'), '[v:val, bufname(v:val)]')";

    NSArray * result = nil;
    @try {
        result = [[principal mediator] evaluateVimExpression:expr];
    } @catch (NSException *ex) { 
        NSLog(@"Exception while evaluating buffer list expr: %@", [ex reason]);
        return;
    }

    [result retain];
    bufferList = result;
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    //NSLog(@"%@ %s", [self className], _cmd);
    return bufferList ? [bufferList count] : 0;
}

- (id)tableView:(NSTableView *)aTableView
    objectValueForTableColumn:(NSTableColumn *)aTableColumn
    row:(int)rowIndex
{
    //NSLog(@"%@ %s", [self className], _cmd);
    NSAssert(bufferList, @"bufferList should be null");
    if ([[aTableColumn identifier] isEqualToString:@"bufnr"]) {
        return [[bufferList objectAtIndex:rowIndex] objectAtIndex:0];
    } else {
        NSString * str = [[bufferList objectAtIndex:rowIndex] objectAtIndex:1];
        if ([str isEqualToString:@""])
            return @"[No Name]";
        else
            return str;
    }
}

- (void)refreshBufferList
{
    [self reloadBufferList];
    NSAssert(bufferListTableView, @"not null");
    [bufferListTableView reloadData];
}

- (void)bufferListTimerFired:(NSTimer *)theTimer
{
    NSLog(@"%@ %s", [self className], _cmd);
    [self refreshBufferList];
}

- (NSView *)topLevelView
{
    return bufferListScrollView;
}

@end
