//
//  FBPrincipal.m
//  FileBrowser
//
//  Created by Matt Tolton on 3/30/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "FBPrincipal.h"
#import "FileSystemItem.h"
#import "BufferListController.h"

static id<PlugInAppMediator> appMediator = nil;

@implementation FBPrincipal

- (void)loadBrowserNib
{
    if (![NSBundle loadNibNamed:@"Browser" owner:self])
        NSLog(@"Error loading Browser nib");
}

- (id)initWithMediator:(id<PlugInInstanceMediator>)instanceMediator
{
    if ((self = [super init]) == nil) return nil;

    NSLog(@"%@ %s", [self className], _cmd);

    mediator = instanceMediator;

    [self loadBrowserNib];


    bufferListTimer = [NSTimer scheduledTimerWithTimeInterval:5.0
                                                       target:bufferListController
                                                     selector:@selector(bufferListTimerFired:)
                                                     userInfo:nil
                                                      repeats:YES];
    [bufferListTimer retain];
    [bufferListController refreshBufferList];

    return self;
}

- (void)dealloc
{
    NSLog(@"%@ %s", [self className], _cmd);

    [bufferListTimer invalidate];
    [bufferListTimer release]; bufferListTimer = nil;

    mediator = nil;

    [browserListController release]; browserListController = nil;
    [bufferListController release]; bufferListController = nil;

    [super dealloc];
}

- (id<PlugInInstanceMediator>)mediator
{
    return mediator;
}

- (void)chooseBrowserDirectory
{
    int result;
    NSOpenPanel * op = [NSOpenPanel openPanel];

    [op setAllowsMultipleSelection:NO];
    [op setCanChooseDirectories:YES];
    [op setCanChooseFiles:NO];

    result = [op runModalForDirectory:NSHomeDirectory() file:nil types:nil];

    if (result == NSOKButton) {
        NSString * filename = [[op filenames] objectAtIndex:0];
        [browserListController changeRootDirectoryTo:filename];
    } 
}

+ (void)chooseBrowserDirectory:(id)sender
{
    id instance = [appMediator keyPlugInInstanceWithClass:[FBPrincipal class]];
    [instance chooseBrowserDirectory];
}

+ (BOOL)validateMenuItem:(NSMenuItem *)item
{
    return YES;
}

+ (BOOL)initializePlugIn:(id<PlugInAppMediator>)mediator 
{
    NSLog(@"%@ %s", [self className], _cmd);
    appMediator = mediator;
    NSMenuItem * item = [[[NSMenuItem alloc] initWithTitle:@"Select File Browser Directory..."
                                                    action:@selector(chooseBrowserDirectory:)
                                             keyEquivalent:@""] autorelease];
    [item setTarget:[FBPrincipal class]];
    [mediator addPlugInMenuItem:item];
    return YES;
}

+ (void)terminatePlugIn 
{
    NSLog(@"%@ %s", [self className], _cmd);
    appMediator = nil;
}

@end
