#import "FileSystemItem.h"

@implementation FileSystemItem
 
#define IsALeafNode ((id)-1)

- (id)initWithPath:(NSString *)path parent:(FileSystemItem *)obj
{
    if (self = [super init])
    {
        relativePath = [[path lastPathComponent] copy];
        parent = obj;
    }
    return self;
}
 
- (id)initWithPath:(NSString *)path 
{
    if ((self = [super init]) == nil) return nil;

    relativePath = [path copy];

    return self;
}
 
// Creates, caches, and returns the array of children
// Loads children incrementally
- (NSArray *)children
{
    if (children == NULL)
    {
        NSFileManager *fileManager = [NSFileManager defaultManager];
        NSString *fullPath = [self fullPath];
        BOOL isDir, valid = [fileManager fileExistsAtPath:fullPath isDirectory:&isDir];
 
        if (valid && isDir)
        {
            NSArray *array = [fileManager directoryContentsAtPath:fullPath];
            int cnt, numChildren = [array count];
            children = [[NSMutableArray alloc] initWithCapacity:numChildren];
 
            for (cnt = 0; cnt < numChildren; cnt++)
            {
                FileSystemItem *newChild = [[FileSystemItem alloc] initWithPath:[array objectAtIndex:cnt] parent:self];
                [children addObject:newChild];
                [newChild release];
            }
        }
        else
        {
             children = IsALeafNode;
        }
    }
    return children;
}
 
 
- (NSString *)relativePath
{
    return relativePath;
}
 
 
- (NSString *)fullPath
{
    // If no parent, return our own relative path
    if (parent == nil) return relativePath;
 
    // recurse up the hierarchy, prepending each parent’s path
    return [[parent fullPath] stringByAppendingPathComponent:relativePath];
}
 
- (FileSystemItem *)childAtIndex:(int)n
{
    return [[self children] objectAtIndex:n];
}
 
- (int)numberOfChildren
{
    id tmp = [self children];
    return (tmp == IsALeafNode) ? (-1) : [tmp count];
}
 
 
- (void)dealloc
{
    if (children != IsALeafNode) [children release];
    [relativePath release];
    [super dealloc];
}
 
@end

